from abc import ABC, abstractmethod
from typing import List
from armorer.profiles import RiskProfile

class BaseInstaller(ABC):
    def __init__(self, profile: RiskProfile, mount_paths: List[str]):
        self.profile = profile
        self.mount_paths = mount_paths

    @abstractmethod
    def install(self):
        """Perform the installation or generation of configuration."""
        pass
