from typing import List, Union
from armorer.agent_sdk import tool
from armorer.installers.schemas import DeploymentPlan

@tool(category="workflow", verification_level="plan_submission")
def submit_deployment_plan(docker_compose: str, intent_checklist: List[str]) -> str:
    """
    Submit the final Deployment Plan for execution.
    
    Args:
        docker_compose: The full content of the docker-compose.yml file.
        intent_checklist: A list of high-level goals for setup (e.g. 'Migrate database').
    """
    # This tool doesn't actually DO anything in the graph execution sense other than
    # getting called so we can capture the arguments.
    # The return value is just for the agent's history.
    return "Deployment Plan submitted successfully. The Executor will now take over."
