from __future__ import annotations

from importlib.metadata import PackageNotFoundError, version as package_version


def get_version(default: str = "0.0.0+dev") -> str:
    try:
        return package_version("armorer")
    except PackageNotFoundError:
        return default

